@@ -29,7 +29,7 @@ class NANONETSOCR():
             print("Authentication Not Successful : Wrong API Key!\n\nGet your free API Key - \n1. Go to https://app.nanonets.com/#/signup?&utm_source=wrapper\n2. On your Nanonets account, Go to Account Info -> API Keys. Generate your free API Key and copy it\n3. Use it to authenticate this Nanonets OCR Wrapper.")
             sys.exit()
         
-    def convert_to_prediction(self, url):
+    def raw_prediction(self, url):
         if 'https://' in url or 'http://' in url:
             payload={'urls': url}
             headers = {}
@@ -41,9 +41,47 @@ class NANONETSOCR():
             response = requests.request("POST", url = self.endpoint, headers=headers, files=files, auth=requests.auth.HTTPBasicAuth(self.token, ''))
             return response.text
         
-    def image_to_string(self, url, formatting = 'none', space_size = 11, line_height = 60, line_threshold = 'low'):
+    def formatting(self, prediction):
+
+        df = pd.DataFrame(prediction)
+        df['vertical'] = df['ymax']
+        df = df[['text','vertical']]
+        df.reset_index(inplace = True)
+        df['on'] = 'on'
+        df = pd.merge(left = df, right = df, on = 'on', how = 'inner')
+        df = df[df['index_y'] - df['index_x'] == 1]
+        df['height'] = abs(df['vertical_y'] - df['vertical_x'])
+        df = df[['text_x', 'text_y', 'height']]
+        df = df[df['height']>20]
+        #line_height = 0.8*df['height'].min()
+        line_height = 0.8*df['height'].nsmallest(11).iloc[-1]
+
+        df = pd.DataFrame(prediction)
+        df['vertical'] = df['ymax']
+        df = df[['text', 'xmin','xmax', 'vertical']]
+        df.reset_index(inplace = True)
+        df['on'] = 'on'
+        df = pd.merge(left = df, right = df, on = 'on', how = 'inner')
+        df = df[df['index_y'] - df['index_x'] == 1]
+        df = df[df['vertical_y'] - df['vertical_x'] <= line_height]
+        df['space'] = df['xmin_y'] - df['xmax_x']
+        df = df[df['space'] > 10]
+        #space_size = 0.8*df['space'].min()
+        space_size = 0.8*df['space'].nsmallest(3).iloc[-1]
+
+        return line_height, space_size
+    
+    def image_to_string(self, url, formatting = 'none', line_threshold = 'low'):
+        line_height = 60
+        space_size = 11
+        try:
+            line_height, space_size = self.formatting(json.loads(self.raw_prediction(url))['results'][0]['page_data'][0]['words'])
+        except:
+            pass
+        
         if formatting == 'none':
-            return json.loads(self.convert_to_prediction(url))['results'][0]['page_data'][0]['raw_text']
+            return json.loads(self.raw_prediction(url))['results'][0]['page_data'][0]['raw_text']
+            
         if formatting == 'lines':
             if line_threshold == 'high':
                 line_height_threshold = 5
@@ -51,21 +89,24 @@ class NANONETSOCR():
                 line_height_threshold = line_height
             lines = []
             line = ''
-            for x in json.loads(self.convert_to_prediction(url))['results'][0]['page_data'][0]['words']:
+            for x in json.loads(self.raw_prediction(url))['results'][0]['page_data'][0]['words']:
 
                 if line == '':
                     line = x['text']
                     y = x['ymax']
+                    prev = x['xmax']
 
                 else:
-                    if abs(x['ymax'] - y) <= line_height_threshold:
+                    if abs(x['ymax'] - y) <= line_height_threshold and x['xmax'] - prev > 0 :
                         line = line + ' ' + x['text']
                         y = x['ymax']
+                        prev = x['xmax']
 
                     else:
                         lines.append(line)
                         line = x['text']
                         y = x['ymax']
+                        prev = x['xmax']
 
 
             lines.append(line)
@@ -82,13 +123,15 @@ class NANONETSOCR():
         if formatting == 'lines and spaces':
             if line_threshold == 'high':
                 line_height_threshold = 5
+                line_height = 60
+                space_size = 11
             if line_threshold == 'low':
                 line_height_threshold = line_height
             lines = []
             spaces = []
             line = ''
             space = 0
-            for x in json.loads(self.convert_to_prediction(url))['results'][0]['page_data'][0]['words']:
+            for x in json.loads(self.raw_prediction(url))['results'][0]['page_data'][0]['words']:
 
                 if line == '':
                     line = x['text']
@@ -99,8 +142,11 @@ class NANONETSOCR():
                     prev = x['xmax']
 
                 else:
-                    if abs(x['ymax'] - y) <= line_height_threshold:
-                        line = line + ' '*round((x['xmin']-prev)/space_size) + x['text']
+                    if abs(x['ymax'] - y) <= line_height_threshold and x['xmax'] - prev > 0 :
+                        numspace = round((x['xmin']-prev)/space_size)
+                        if numspace <=2:
+                            numspace = 1
+                        line = line + ' '*numspace + x['text']
                         y = x['ymax']
                         y2 = x['ymin']
                         prev = x['xmax']
@@ -140,154 +186,10 @@ class NANONETSOCR():
             
             return result
 
-    def image_to_boxes(self, url):
-        return json.loads(self.convert_to_prediction(url))['results'][0]['page_data'][0]['words']
-        
-    def image_to_searchablepdf(self, url, filename = 'did not give a filename', formatting = 'lines and spaces',space_size = 11, line_height = 60, line_threshold = 'low'):
-        text = self.image_to_string(url, formatting = formatting, space_size = space_size, line_height = line_height, line_threshold = line_threshold)
-        if filename == 'did not give a filename':
-            filename = url + '.pdf'
-        a4_width_mm = 600
-        pt_to_mm = 0.35
-        fontsize_pt = 10
-        fontsize_mm = fontsize_pt * pt_to_mm
-        margin_bottom_mm = 10
-        character_width_mm = 7 * pt_to_mm
-        width_text = a4_width_mm / character_width_mm
-
-        pdf = FPDF(orientation='P', unit='mm', format='A4')
-        pdf.set_auto_page_break(True, margin=margin_bottom_mm)
-        pdf.add_page()
-        pdf.set_font(family='Courier', size=fontsize_pt)
-        splitted = text.split('\n')
-
-        for line in splitted:
-            lines = textwrap.wrap(line, width_text)
-
-            if len(lines) == 0:
-                pdf.ln()
-
-            for wrap in lines:
-                pdf.multi_cell(0, fontsize_mm, wrap)
-
-        pdf.output(filename, 'F')
-        print(filename + ' created in current directory')
-        
-    def image_to_csv(self, url, filename = 'did not give a filename', cell_width = 250, cell_height = 50, is_table=True):
-        if is_table == False :
-            lines = []
-            spaces = []
-            line = ''
-            space = 0
-            if filename == 'did not give a filename':
-                filename = url + '.csv'
-            for x in json.loads(self.convert_to_prediction(url))['results'][0]['page_data'][0]['words']:
-
-                if line == '':
-                    line = x['text']
-                    y = x['ymax']
-                    y2 = x['ymin']
-                    space = x['xmin']
-                    spaces.append(space)
-                    cell = (x['xmin'] + x['xmax']) / cell_width
-
-                else:
-                    if abs(x['ymax'] - y) <=5:
-                        if round(((x['xmin'] + x['xmax']) / cell_width) - cell) == 0 :
-                            line = line + ' ' + x['text']
-                        else :
-                            line = line + ','*round(((x['xmin'] + x['xmax']) / cell_width) - cell) + x['text']
-                        y = x['ymax']
-                        y2 = x['ymin']
-                        cell = (x['xmin'] + x['xmax']) / cell_width
-
-                    else:
-                        lines.append(line)
-                        while x['ymax'] - y2 > cell_height:
-                            lines.append(",")
-                            y2 = y2 + cell_height
-
-                        line = x['text']
-                        y = x['ymax']
-                        y2 = x['ymin']
-                        cell = (x['xmin'] + x['xmax']) / cell_width
-                        space = x['xmin']
-                        spaces.append(space)
-
-            lines.append(line)
-
-            with open(filename, "w", newline="") as f:
-                writer = csv.writer(f)
-                for row in lines:
-                    writer.writerow(row.split(','))
-
-            print(filename + ' created in current directory')
-        
-        if is_table == True:
-            if filename == 'did not give a filename':
-                filename = url + '.csv'
-            
-            data = {'file': open(url, 'rb')}
-            
-            url = 'https://app.nanonets.com/api/v2/OCR/Model/6aace346-9b80-4c02-8fd4-138c7575382c/LabelFile/'
-            
-            response = requests.post(url, auth=requests.auth.HTTPBasicAuth('5_no1N0KdX4Eac_e8BnxDzdsY6ycz_rk', ''), files=data)
-            
-            i = 1
-            
-            if not(len(response.json()["result"]) == 1 and len(response.json()["result"][0]["prediction"]) ==1):
-                with open(filename, "w") as my_empty_csv:
-                    pass
-                
-                for page in response.json()["result"]:
-                    for table in page["prediction"]:
-
-                        maxrow = 0
-                        maxcol = 0
-
-                        for el in table['cells']:
-                            maxrow = max(el['row'],maxrow)
-                            maxcol = max(el['col'], maxcol)
-
-                        df = pd.DataFrame(np.zeros([maxrow, maxcol])*np.nan)
-
-                        for el in table['cells']:
-                            row = el['row'] - 1
-                            col = el['col'] - 1
-                            df.iloc[row,col] = el['text']
-
-                        number = i
-                        i = i + 1
-                        tablelist = ['',''],["Table "+str(number), '']
-                        tablenumber = pd.DataFrame(data = tablelist)
-                        tablenumber.to_csv(filename,index=False, header=False, mode='a')
-                        df.to_csv(filename,index=False, header=False, mode='a')
-                print(filename + ' created in current directory')
-                
-            else:
-                
-                table = response.json()["result"][0]["prediction"][0]
-                maxrow = 0
-                maxcol = 0
-
-                for el in table['cells']:
-                    maxrow = max(el['row'],maxrow)
-                    maxcol = max(el['col'], maxcol)
-
-                df = pd.DataFrame(np.zeros([maxrow, maxcol])*np.nan)
-
-                for el in table['cells']:
-                    row = el['row'] - 1
-                    col = el['col'] - 1
-                    df.iloc[row,col] = el['text']
-
-                df.to_csv(filename,index=False, header=False)
-                print(filename + ' created in current directory')
-            
-    def pdf_to_string(self, url, formatting = 'none', space_size = 11, line_height = 50, line_threshold = 'low'):
+    def pdf_to_string(self, url, formatting = 'none', line_threshold = 'low'):
         if formatting == 'none':
             result = ''
-            for el in json.loads(self.convert_to_prediction(url))['results'][0]['page_data']:
+            for el in json.loads(self.raw_prediction(url))['results'][0]['page_data']:
                 if result == '':
                     result = el['raw_text']
                 else:
@@ -296,17 +198,26 @@ class NANONETSOCR():
         
         if formatting == 'pages':
             result = []
-            for el in json.loads(self.convert_to_prediction(url))['results'][0]['page_data']:
+            for el in json.loads(self.raw_prediction(url))['results'][0]['page_data']:
                 result.append(el['raw_text'])
             return result
         
         if formatting == 'lines':
-            if line_threshold == 'high':
-                line_height_threshold = 5
-            if line_threshold == 'low':
-                line_height_threshold = line_height
             pages = []
-            for el in json.loads(self.convert_to_prediction(url))['results'][0]['page_data']:    
+            for el in json.loads(self.raw_prediction(url))['results'][0]['page_data']:
+
+                line_height = 60
+                space_size = 11
+                try:
+                    line_height, space_size = self.formatting(el['words'])
+                except:
+                    pass
+                
+                if line_threshold == 'high':
+                    line_height_threshold = 5
+                if line_threshold == 'low':
+                    line_height_threshold = line_height
+
                 lines = []
                 line = ''
                 for x in el['words']:
@@ -314,16 +225,19 @@ class NANONETSOCR():
                     if line == '':
                         line = x['text']
                         y = x['ymax']
+                        prev = x['xmax']
 
                     else:
-                        if abs(x['ymax'] - y) <= line_height_threshold:
+                        if abs(x['ymax'] - y) <= line_height_threshold and x['xmax'] - prev > 0 :
                             line = line + ' ' + x['text']
                             y = x['ymax']
+                            prev = x['xmax']
 
                         else:
                             lines.append(line)
                             line = x['text']
                             y = x['ymax']
+                            prev = x['xmax']
 
 
                 lines.append(line)
@@ -339,12 +253,23 @@ class NANONETSOCR():
             return pages
         
         if formatting == 'lines and spaces' :
-            if line_threshold == 'high':
-                line_height_threshold = 5
-            if line_threshold == 'low':
-                line_height_threshold = line_height
             pages = []
-            for el in json.loads(self.convert_to_prediction(url))['results'][0]['page_data']:    
+            for el in json.loads(self.raw_prediction(url))['results'][0]['page_data']:    
+
+                line_height = 60
+                space_size = 11
+                try:
+                    line_height, space_size = self.formatting(el['words'])
+                except:
+                    pass
+                
+                if line_threshold == 'high':
+                    line_height_threshold = 5
+                    line_height = 60
+                    space_size = 11
+                if line_threshold == 'low':
+                    line_height_threshold = line_height
+
                 lines = []
                 spaces = []
                 line = ''
@@ -360,8 +285,11 @@ class NANONETSOCR():
                         prev = x['xmax']
                         
                     else:
-                        if abs(x['ymax'] - y) <= line_height_threshold:
-                            line = line + ' '*round((x['xmin']-prev)/space_size) + x['text']
+                        if abs(x['ymax'] - y) <= line_height_threshold and x['xmax'] - prev > 0 :
+                            numspace = round((x['xmin']-prev)/space_size)
+                            if numspace <=2:
+                                numspace = 1
+                            line = line + ' '*numspace + x['text']
                             y = x['ymax']
                             y2 = x['ymin']
                             prev = x['xmax']
@@ -400,166 +328,16 @@ class NANONETSOCR():
                 pages.append(result)
             return pages  
     
+    def image_to_boxes(self, url):
+        return json.loads(self.raw_prediction(url))['results'][0]['page_data'][0]['words']
+    
     def pdf_to_boxes(self, url):
         pages = []
-        for el in json.loads(self.convert_to_prediction(url))['results'][0]['page_data']:
+        for el in json.loads(self.raw_prediction(url))['results'][0]['page_data']:
             result = el['words']
             pages.append(result)
         return pages
     
-    def pdf_to_searchablepdf(self, url, filename = 'did not give a filename', formatting = 'lines and spaces',space_size = 11, line_height = 50, line_threshold = 'low'):
-        texts = self.pdf_to_string(url, formatting = formatting, space_size = space_size, line_height = line_height, line_threshold = line_threshold)
-        if filename == 'did not give a filename':
-            filename = url + '.pdf'
-        a4_width_mm = 600
-        pt_to_mm = 0.35
-        fontsize_pt = 10
-        fontsize_mm = fontsize_pt * pt_to_mm
-        margin_bottom_mm = 10
-        character_width_mm = 7 * pt_to_mm
-        width_text = a4_width_mm / character_width_mm
-
-        pdf = FPDF(orientation='P', unit='mm', format='A4')
-        pdf.set_auto_page_break(False, margin=margin_bottom_mm)
-        pdf.set_font(family='Courier', size=fontsize_pt)
-        
-        text = 'this is starting point'
-        for text in texts:
-            
-            pdf.add_page()
-            splitted = text.split('\n')
-
-            for line in splitted:
-                lines = textwrap.wrap(line, width_text)
-
-                if len(lines) == 0:
-                    pdf.ln()
-
-                for wrap in lines:
-                    pdf.multi_cell(0, fontsize_mm, wrap)
-
-        pdf.output(filename, 'F')
-        print(filename + ' created in current directory')
-                
-    def pdf_to_csv(self, url, filename = 'did not give a filename', cell_width = 750, cell_height = 150, is_table = True):
-        if is_table==False:
-            pages = []
-            if filename == 'did not give a filename':
-                filename = url + '.csv'
-            for el in json.loads(self.convert_to_prediction(url))['results'][0]['page_data']:    
-
-                lines = []
-                spaces = []
-                line = ''
-                space = 0
-
-                for x in el['words']:
-
-                    if line == '':
-                        line = x['text']
-                        y = x['ymax']
-                        y2 = x['ymin']
-                        space = x['xmin']
-                        spaces.append(space)
-                        cell = (x['xmin'] + x['xmax']) / cell_width
-
-                    else:
-                        if abs(x['ymax'] - y) <=5:
-                            if round(((x['xmin'] + x['xmax']) / cell_width) - cell) == 0 :
-                                line = line + ' ' + x['text']
-                            else :
-                                line = line + ','*round(((x['xmin'] + x['xmax']) / cell_width) - cell) + x['text']
-                            y = x['ymax']
-                            y2 = x['ymin']
-                            cell = (x['xmin'] + x['xmax']) / cell_width
-
-                        else:
-                            lines.append(line)
-                            while x['ymax'] - y2 > cell_height:
-                                lines.append(",")
-                                y2 = y2 + cell_height
-
-                            line = x['text']
-                            y = x['ymax']
-                            y2 = x['ymin']
-                            cell = (x['xmin'] + x['xmax']) / cell_width
-                            space = x['xmin']
-                            spaces.append(space)
-
-                lines.append(line)
-                pages.append(lines)
-
-            with open(filename, "w", newline="") as f:
-                writer = csv.writer(f)
-                for page in pages:
-                    for row in page:
-                        writer.writerow(row.split(','))
-                    writer.writerow([""])
-
-            print(filename + ' created in current directory')
-        
-        if is_table == True:
-            
-            if filename == 'did not give a filename':
-                filename = url + '.csv'
-            
-            data = {'file': open(url, 'rb')}
-            
-            url = 'https://app.nanonets.com/api/v2/OCR/Model/6aace346-9b80-4c02-8fd4-138c7575382c/LabelFile/'
-            
-            response = requests.post(url, auth=requests.auth.HTTPBasicAuth('5_no1N0KdX4Eac_e8BnxDzdsY6ycz_rk', ''), files=data)
-            
-            i = 1
-            
-            if not(len(response.json()["result"]) == 1 and len(response.json()["result"][0]["prediction"]) ==1):
-                with open(filename, "w") as my_empty_csv:
-                    pass
-                
-                for page in response.json()["result"]:
-                    for table in page["prediction"]:
-
-                        maxrow = 0
-                        maxcol = 0
-
-                        for el in table['cells']:
-                            maxrow = max(el['row'],maxrow)
-                            maxcol = max(el['col'], maxcol)
-
-                        df = pd.DataFrame(np.zeros([maxrow, maxcol])*np.nan)
-
-                        for el in table['cells']:
-                            row = el['row'] - 1
-                            col = el['col'] - 1
-                            df.iloc[row,col] = el['text']
-
-                        number = i
-                        i = i + 1
-                        tablelist = ['',''],["Table "+str(number), '']
-                        tablenumber = pd.DataFrame(data = tablelist)
-                        tablenumber.to_csv(filename,index=False, header=False, mode='a')
-                        df.to_csv(filename,index=False, header=False, mode='a')
-                print(filename + ' created in current directory')
-                
-            else:
-                
-                table = response.json()["result"][0]["prediction"][0]
-                maxrow = 0
-                maxcol = 0
-
-                for el in table['cells']:
-                    maxrow = max(el['row'],maxrow)
-                    maxcol = max(el['col'], maxcol)
-
-                df = pd.DataFrame(np.zeros([maxrow, maxcol])*np.nan)
-
-                for el in table['cells']:
-                    row = el['row'] - 1
-                    col = el['col'] - 1
-                    df.iloc[row,col] = el['text']
-
-                df.to_csv(filename,index=False, header=False)
-                print(filename + ' created in current directory')
-    
     def extract_tables(self, file_path, download = False, output_file_name = 'did not give a filename'):
         if self.authentication==1:
             if download == False:
@@ -581,7 +359,7 @@ class NANONETSOCR():
             print("Authentication Not Successful : Wrong API Key!\n\nGet your free API Key - \n1. Go to https://app.nanonets.com/#/signup?&utm_source=wrapper\n2. On your Nanonets account, Go to Account Info -> API Keys. Generate your free API Key and copy it\n3. Use it to authenticate this Nanonets OCR Wrapper.")
             sys.exit()
             
-    def searchable_pdf(self, file_path, output_file_name = 'did not give a filename'):
+    def convert_to_searchable_pdf(self, file_path, output_file_name = 'did not give a filename'):
         if self.authentication==1:
             if output_file_name == 'did not give a filename':
                 output_file_name = str(file_path) + '_searchable.pdf'
@@ -596,14 +374,42 @@ class NANONETSOCR():
             print("Authentication Not Successful : Wrong API Key!\n\nGet your free API Key - \n1. Go to https://app.nanonets.com/#/signup?&utm_source=wrapper\n2. On your Nanonets account, Go to Account Info -> API Keys. Generate your free API Key and copy it\n3. Use it to authenticate this Nanonets OCR Wrapper.")
             sys.exit()
 
+    def convert_to_prediction(self, file_path):
+        return json.loads(self.raw_prediction(file_path))
+    
     def convert_to_boxes(self, file_path):
         if '.pdf' in file_path:
             return self.pdf_to_boxes(file_path)
         else :
             return self.image_to_boxes(file_path)
     
-    def convert_to_string(self, file_path, formatting = 'none', line_threshold = 'low'):
+    def convert_to_string(self, file_path, formatting = 'lines and spaces', line_threshold = 'low'):
         if '.pdf' in file_path:
-            return self.pdf_to_string(file_path, formatting = formatting, line_threshold = line_threshold)
+            result = self.pdf_to_string(file_path, formatting = formatting, line_threshold = line_threshold)
+            if formatting == 'none':
+                return result
+            else:
+                string = ''
+                i = 1
+                for page in result:
+                    string = string + "PAGE " + str(i) + '\n\n' + page + '\n\n'
+                    i = i + 1
+                return string
         else :
             return self.image_to_string(file_path, formatting = formatting, line_threshold = line_threshold)
+
+    def convert_to_csv(self, file_path, output_file_name = 'did not give a filename'):
+        self.extract_tables(file_path = file_path, download = True, output_file_name = output_file_name)
+
+    def convert_to_tables(self, file_path):
+        result = self.extract_tables(file_path = file_path)
+        return result
+    
+    def convert_to_txt(self, file_path, formatting = 'lines and spaces', line_threshold = 'low', output_file_name = 'did not give a filename'):
+        string = self.convert_to_string(file_path = file_path, formatting = formatting, line_threshold = line_threshold)
+        if output_file_name == 'did not give a filename':
+            output_file_name = str(file_path) + '_text.txt'
+        textfile = open(output_file_name, 'w')
+        textfile.write(string)
+        textfile.close()
+        
\ No newline at end of file
